import Taro, { useReachBottom } from "@tarojs/taro";
import { View, Text } from "@tarojs/components";

import NavBar from "@/components/nav-bar/index";
import SearchBar from "@/components/search-bar/index";
import TabPageWrapper from "@/components/tab-page-wrapper/index";
import style from "./index.module.less";
import { useEffect, useState } from "react";
import ContactCard from "./components/contact-card/index";
import { getContactList, searchContact } from "@/service/contact";
import { IContactModel } from "@/types/index";
import { useDebounceWithParams } from "@/utils/index";
import { reportPageVisit } from "@/utils/report";
import { useLoadMore } from "@/utils/loadMore";

export default function Index() {
  const [searchValue, setSearchValue] = useState("");
  const [list, setList] = useState<IContactModel[]>([]);

  const fetcher = async ([_url, page, pageSize]) => {
    const res = await getContactList({ page, pageSize });
    if (res.code === 0 && res.data) {
      return res.data;
    }
  };
  const { data, loadMore, refetch } = useLoadMore<IContactModel[]>({
    url: "/v1/contact",
    fetcher,
  });

  // 搜索
  const debounceSearch = useDebounceWithParams(async (searchText) => {
    if (searchText === "") {
      console.log("searchValue is empty", searchText);
      return;
    }
    const res = await searchContact(searchText);
    if (res.code === 0 && res.data) {
      setList(res.data);
    } else {
      setList([]);
    }
  }, 500);

  const resetFetchList = () => {
    setList([]);
    refetch();
  };

  const handleSearchBarChanged = (v: string) => {
    setSearchValue(v);
    if (!v.length) {
      resetFetchList();
      return;
    }

    Taro.nextTick(() => {
      debounceSearch(v);
    });
  };

  const handleClear = () => {
    setSearchValue("");
    resetFetchList();
  };

  useReachBottom(() => {
    // 加载更多数据
    // 如果搜索框中有数据由不加载更多数据
    if (searchValue.length) {
      return;
    }
    loadMore();
  });

  useEffect(() => {
    if (data?.length) {
      setList((prevdata) => {
        return [...prevdata, ...data];
      });
    }
  }, [data]);

  reportPageVisit();

  const renderContent = () => {
    if (list?.length) {
      return list.map((item) => (
        <ContactCard
          refresh={resetFetchList}
          deleteable
          key={item.contactProfileId}
          data={item}
          fromContact
        ></ContactCard>
      ));
    }
    return (
      <View className="flex flex-col pt-56 items-center">
        <View className="data-empty"></View>
        <View className="text-12 text-gray-45 text-center leading-24 mt-12">
          暂无数据
        </View>
      </View>
    );
  };

  return (
    <TabPageWrapper>
      <NavBar>
        <View className={style.titleMarkup}></View>
        <Text>联系人</Text>
      </NavBar>
      <View className="flex flex-col gap-20 w-full">
        <SearchBar
          value={searchValue}
          onChange={(e) => handleSearchBarChanged(e)}
          onClear={() => handleClear()}
        ></SearchBar>
        <View className="flex flex-col gap-8 w-full">{renderContent()}</View>
      </View>
    </TabPageWrapper>
  );
}
